@@ -9,7 +9,7 @@
 
 #include "driver/dcomputecodegenerator.h"
 #include "driver/cl_options.h"
-#include "ddmd/errors.h"
+#include "dmd/errors.h"
 #include "gen/cl_helpers.h"
 #include "ir/irdsymbol.h"
 #include "llvm/Support/CommandLine.h"
