@@ -9,8 +9,10 @@
 
 #include "errors.h"
 #include "driver/cl_options.h"
+#include "driver/cl_options_instrumentation.h"
 #include "driver/cl_options_sanitizers.h"
 #include "driver/exe_path.h"
+#include "driver/ldc-version.h"
 #include "driver/tool.h"
 #include "gen/irstate.h"
 #include "gen/logger.h"
@@ -47,10 +49,12 @@ public:
              llvm::cl::boolOrDefault fullyStaticFlag);
 
 private:
-  virtual void addSanitizers();
-  virtual void addASanLinkFlags();
-  virtual void addFuzzLinkFlags();
-  virtual void addCppStdlibLinkFlags();
+  virtual void addSanitizers(const llvm::Triple &triple);
+  virtual void addASanLinkFlags(const llvm::Triple &triple);
+  virtual void addFuzzLinkFlags(const llvm::Triple &triple);
+  virtual void addCppStdlibLinkFlags(const llvm::Triple &triple);
+  virtual void addProfileRuntimeLinkFlags(const llvm::Triple &triple);
+  virtual void addXRayLinkFlags(const llvm::Triple &triple);
 
   virtual void addLinker();
   virtual void addUserSwitches();
@@ -184,25 +188,57 @@ void ArgsBuilder::addLTOLinkFlags() {
 
 // Returns the arch name as used in the compiler_rt libs.
 // FIXME: implement correctly for non-x86 platforms (e.g. ARM)
-llvm::StringRef getCompilerRTArchName() {
-  return global.params.targetTriple->getArchName();
+// See clang/lib/Driver/Toolchain.cpp.
+llvm::StringRef getCompilerRTArchName(const llvm::Triple &triple) {
+  return triple.getArchName();
 }
 
 // Returns the libname as full path and with arch suffix and extension.
 // For example, with name="libldc_rt.fuzzer", the returned string is
 // "libldc_rt.fuzzer_osx.a" on Darwin.
-std::string getFullCompilerRTLibPath(llvm::StringRef name,
+std::string getFullCompilerRTLibPath(const llvm::Triple &triple,
+                                     llvm::StringRef name,
                                      bool sharedLibrary = false) {
-  if (global.params.targetTriple->isOSDarwin()) {
+  if (triple.isOSDarwin()) {
     return exe_path::prependLibDir(
         name + (sharedLibrary ? "_osx_dynamic.dylib" : "_osx.a"));
   } else {
-    return exe_path::prependLibDir(name + "-" + getCompilerRTArchName() +
+    return exe_path::prependLibDir(name + "-" + getCompilerRTArchName(triple) +
                                    (sharedLibrary ? ".so" : ".a"));
   }
 }
 
-void ArgsBuilder::addASanLinkFlags() {
+// Clang's RT libs are in a subdir of the lib dir.
+// Returns the libname as full path and with arch suffix and extension.
+// For example, with name="libclang_rt.asan" and sharedLibrary=true, the
+// returned string is
+// "clang/6.0.0/lib/darwin/libclang_rt.asan_osx_dynamic.dylib" on Darwin.
+// This function is "best effort", the path may not be what Clang does...
+// See clang/lib/Driver/Toolchain.cpp.
+std::string getFullClangCompilerRTLibPath(const llvm::Triple &triple,
+                                          llvm::StringRef name,
+                                          bool sharedLibrary = false) {
+  llvm::StringRef OSName =
+      triple.isOSDarwin()
+          ? "darwin"
+          : triple.isOSFreeBSD() ? "freebsd" : triple.getOSName();
+
+  std::string relPath = (llvm::Twine("clang/") + ldc::llvm_version_base +
+                         "/lib/" + OSName + "/" + name)
+                            .str();
+
+  if (triple.isOSDarwin()) {
+    return exe_path::prependLibDir(
+        llvm::Twine(relPath) +
+        (sharedLibrary ? "_osx_dynamic.dylib" : "_osx.a"));
+  } else {
+    return exe_path::prependLibDir(llvm::Twine(relPath) + "-" +
+                                   getCompilerRTArchName(triple) +
+                                   (sharedLibrary ? ".so" : ".a"));
+  }
+}
+
+void ArgsBuilder::addASanLinkFlags(const llvm::Triple &triple) {
   // Examples: "libclang_rt.asan-x86_64.a" or "libclang_rt.asan-arm.a" and
   // "libclang_rt.asan-x86_64.so"
 
@@ -210,14 +246,19 @@ void ArgsBuilder::addASanLinkFlags() {
   // In case of shared ASan, I think we also need to statically link with
   // libclang_rt.asan-preinit-<arch>.a on Linux. On Darwin, the only option is
   // to use the shared library.
-  bool linkSharedASan = global.params.targetTriple->isOSDarwin();
+  bool linkSharedASan = triple.isOSDarwin();
   std::string searchPaths[] = {
-      getFullCompilerRTLibPath("libldc_rt.asan", linkSharedASan),
-      getFullCompilerRTLibPath("libclang_rt.asan", linkSharedASan),
+      getFullCompilerRTLibPath(triple, "libldc_rt.asan", linkSharedASan),
+      getFullCompilerRTLibPath(triple, "libclang_rt.asan", linkSharedASan),
+      getFullClangCompilerRTLibPath(triple, "libclang_rt.asan", linkSharedASan),
   };
 
   for (const auto &filepath : searchPaths) {
-    if (llvm::sys::fs::exists(filepath)) {
+    IF_LOG Logger::println("Searching ASan lib: %s", filepath.c_str());
+
+    if (llvm::sys::fs::exists(filepath) &&
+        !llvm::sys::fs::is_directory(filepath)) {
+      IF_LOG Logger::println("Found, linking with %s", filepath.c_str());
       args.push_back(filepath);
 
       if (linkSharedASan) {
@@ -245,11 +286,12 @@ void ArgsBuilder::addASanLinkFlags() {
 
 // Adds all required link flags for -fsanitize=fuzzer when libFuzzer library is
 // found.
-void ArgsBuilder::addFuzzLinkFlags() {
+void ArgsBuilder::addFuzzLinkFlags(const llvm::Triple &triple) {
   std::string searchPaths[] = {
 #if LDC_LLVM_VER >= 600
-    getFullCompilerRTLibPath("libldc_rt.fuzzer"),
-    getFullCompilerRTLibPath("libclang_rt.fuzzer"),
+    getFullCompilerRTLibPath(triple, "libldc_rt.fuzzer"),
+    getFullCompilerRTLibPath(triple, "libclang_rt.fuzzer"),
+    getFullClangCompilerRTLibPath(triple, "libclang_rt.fuzzer"),
 #else
     exe_path::prependLibDir("libFuzzer.a"),
     exe_path::prependLibDir("libLLVMFuzzer.a"),
@@ -257,24 +299,54 @@ void ArgsBuilder::addFuzzLinkFlags() {
   };
 
   for (const auto &filepath : searchPaths) {
-    if (llvm::sys::fs::exists(filepath)) {
+    IF_LOG Logger::println("Searching libFuzzer: %s", filepath.c_str());
+
+    if (llvm::sys::fs::exists(filepath) &&
+        !llvm::sys::fs::is_directory(filepath)) {
+      IF_LOG Logger::println("Found, linking with %s", filepath.c_str());
       args.push_back(filepath);
 
       // libFuzzer requires the C++ std library, but only add the link flags
       // when libFuzzer was found.
-      addCppStdlibLinkFlags();
+      addCppStdlibLinkFlags(triple);
       return;
     }
   }
 }
 
-void ArgsBuilder::addCppStdlibLinkFlags() {
+// Adds all required link flags for -fxray-instrument when the xray library is
+// found.
+void ArgsBuilder::addXRayLinkFlags(const llvm::Triple &triple) {
+  std::string searchPaths[] = {
+    getFullCompilerRTLibPath(triple, "libldc_rt.xray"),
+    getFullCompilerRTLibPath(triple, "libclang_rt.xray"),
+    getFullClangCompilerRTLibPath(triple, "libclang_rt.xray"),
+  };
+
+  bool linkerDarwin = triple.isOSDarwin();
+  if (!triple.isOSLinux())
+    warning(Loc(), "XRay is not (fully) supported on non-Linux target OS.");
+
+  for (const auto &filepath : searchPaths) {
+    if (!linkerDarwin)
+      addLdFlag("--whole-archive");
+
+    args.push_back(filepath);
+
+    if (!linkerDarwin)
+      addLdFlag("--no-whole-archive");
+
+    return;
+  }
+}
+
+void ArgsBuilder::addCppStdlibLinkFlags(const llvm::Triple &triple) {
   if (linkNoCpp)
     return;
 
-  switch (global.params.targetTriple->getOS()) {
+  switch (triple.getOS()) {
   case llvm::Triple::Linux:
-    if (global.params.targetTriple->getEnvironment() == llvm::Triple::Android) {
+    if (triple.getEnvironment() == llvm::Triple::Android) {
       args.push_back("-lc++");
     } else {
       args.push_back("-lstdc++");
@@ -297,13 +369,40 @@ void ArgsBuilder::addCppStdlibLinkFlags() {
   }
 }
 
-void ArgsBuilder::addSanitizers() {
+// Adds all required link flags for PGO.
+void ArgsBuilder::addProfileRuntimeLinkFlags(const llvm::Triple &triple) {
+  std::string searchPaths[] = {
+    getFullCompilerRTLibPath(triple, "libldc_rt.profile"),
+    getFullCompilerRTLibPath(triple, "libclang_rt.profile"),
+    getFullClangCompilerRTLibPath(triple, "libclang_rt.profile"),
+  };
+
+#if LDC_LLVM_VER >= 308
+  if (global.params.targetTriple->isOSLinux()) {
+    // For Linux, explicitly define __llvm_profile_runtime as undefined
+    // symbol, so that the initialization part of profile-rt is linked in.
+    addLdFlag("-u", llvm::getInstrProfRuntimeHookVarName());
+  }
+#endif
+
+  for (const auto &filepath : searchPaths) {
+    IF_LOG Logger::println("Searching profile runtime: %s", filepath.c_str());
+
+    if (llvm::sys::fs::exists(filepath)) {
+      IF_LOG Logger::println("Found, linking with %s", filepath.c_str());
+      args.push_back(filepath);
+      return;
+    }
+  }
+}
+
+void ArgsBuilder::addSanitizers(const llvm::Triple &triple) {
   if (opts::isSanitizerEnabled(opts::AddressSanitizer)) {
-    addASanLinkFlags();
+    addASanLinkFlags(triple);
   }
 
   if (opts::isSanitizerEnabled(opts::FuzzSanitizer)) {
-    addFuzzLinkFlags();
+    addFuzzLinkFlags(triple);
   }
 
   // TODO: instead of this, we should link with our own sanitizer libraries
@@ -324,22 +423,13 @@ void ArgsBuilder::addSanitizers() {
 void ArgsBuilder::build(llvm::StringRef outputPath,
                         llvm::cl::boolOrDefault fullyStaticFlag) {
   // object files
-  for (auto objfile : *global.params.objfiles) {
+  for (auto objfile : global.params.objfiles) {
     args.push_back(objfile);
   }
 
   // Link with profile-rt library when generating an instrumented binary.
-  // profile-rt uses Phobos (MD5 hashing) and therefore must be passed on the
-  // commandline before Phobos.
-  if (global.params.genInstrProf) {
-#if LDC_LLVM_VER >= 308
-    if (global.params.targetTriple->isOSLinux()) {
-      // For Linux, explicitly define __llvm_profile_runtime as undefined
-      // symbol, so that the initialization part of profile-rt is linked in.
-      addLdFlag("-u", llvm::getInstrProfRuntimeHookVarName());
-    }
-#endif
-    args.push_back("-lldc-profile-rt");
+  if (opts::isInstrumentingForPGO()) {
+    addProfileRuntimeLinkFlags(*global.params.targetTriple);
   }
 
   if (opts::enableDynamicCompile) {
@@ -348,9 +438,12 @@ void ArgsBuilder::build(llvm::StringRef outputPath,
   }
 
   // user libs
-  for (auto libfile : *global.params.libfiles) {
+  for (auto libfile : global.params.libfiles) {
     args.push_back(libfile);
   }
+  for (auto dllfile : global.params.dllfiles) {
+    args.push_back(dllfile);
+  }
 
   if (global.params.dll) {
     args.push_back("-shared");
@@ -363,7 +456,11 @@ void ArgsBuilder::build(llvm::StringRef outputPath,
   args.push_back("-o");
   args.push_back(outputPath);
 
-  addSanitizers();
+  addSanitizers(*global.params.targetTriple);
+
+  if (opts::fXRayInstrument) {
+    addXRayLinkFlags(*global.params.targetTriple);
+  }
 
 #if LDC_LLVM_VER >= 309
   // Add LTO link flags before adding the user link switches, such that the user
@@ -376,7 +473,7 @@ void ArgsBuilder::build(llvm::StringRef outputPath,
   addUserSwitches();
 
   // libs added via pragma(lib, libname)
-  for (auto ls : *global.params.linkswitches) {
+  for (auto ls : global.params.linkswitches) {
     args.push_back(ls);
   }
 
@@ -385,7 +482,7 @@ void ArgsBuilder::build(llvm::StringRef outputPath,
     // instrumented binary. The runtime relies on magic sections, which
     // would be stripped by gc-section on older version of ld, see bug:
     // https://sourceware.org/bugzilla/show_bug.cgi?id=19161
-    if (!opts::disableLinkerStripDead && !global.params.genInstrProf) {
+    if (!opts::disableLinkerStripDead && !opts::isInstrumentingForPGO()) {
       addLdFlag("--gc-sections");
     }
   }
@@ -440,10 +537,12 @@ void ArgsBuilder::addUserSwitches() {
 void ArgsBuilder::addDefaultLibs() {
   bool addSoname = false;
 
-  switch (global.params.targetTriple->getOS()) {
+  const auto &triple = *global.params.targetTriple;
+
+  switch (triple.getOS()) {
   case llvm::Triple::Linux:
     addSoname = true;
-    if (global.params.targetTriple->getEnvironment() == llvm::Triple::Android) {
+    if (triple.getEnvironment() == llvm::Triple::Android) {
       args.push_back("-ldl");
       args.push_back("-lm");
       break;
@@ -476,7 +575,7 @@ void ArgsBuilder::addDefaultLibs() {
     break;
   }
 
-  if (global.params.targetTriple->isWindowsGNUEnvironment()) {
+  if (triple.isWindowsGNUEnvironment()) {
     // This is really more of a kludge, as linking in the Winsock functions
     // should be handled by the pragma(lib, ...) in std.socket, but it
     // makes LDC behave as expected for now.
@@ -498,7 +597,7 @@ void ArgsBuilder::addTargetFlags() {
 // (Yet unused) specialization for plain ld.
 
 class LdArgsBuilder : public ArgsBuilder {
-  void addSanitizers() override {}
+  void addSanitizers(const llvm::Triple &triple) override {}
 
   void addLinker() override {}
 
