@@ -188,7 +188,7 @@ static Type *DtoArrayElementType(Type *arrayType) {
 static void copySlice(Loc &loc, LLValue *dstarr, LLValue *sz1, LLValue *srcarr,
                       LLValue *sz2, bool knownInBounds) {
   const bool checksEnabled =
-      global.params.useAssert || gIR->emitArrayBoundsChecks();
+      global.params.useAssert == CHECKENABLEon || gIR->emitArrayBoundsChecks();
   if (checksEnabled && !knownInBounds) {
     LLValue *fn = getRuntimeFunction(loc, gIR->module, "_d_array_slice_copy");
     gIR->CreateCallOrInvoke(fn, dstarr, sz1, srcarr, sz2);
@@ -799,33 +799,34 @@ DSliceValue *DtoResizeDynArray(Loc &loc, Type *arrayType, DValue *array,
 
 ////////////////////////////////////////////////////////////////////////////////
 
-void DtoCatAssignElement(Loc &loc, Type *arrayType, DValue *array,
-                         Expression *exp) {
+void DtoCatAssignElement(Loc &loc, DValue *array, Expression *exp) {
   IF_LOG Logger::println("DtoCatAssignElement");
   LOG_SCOPE;
 
   assert(array);
 
-  LLValue *oldLength = DtoArrayLen(array);
+  Type *arrayType = array->type->toBasetype();
 
-  // Do not move exp->toElem call after creating _d_arrayappendcTX,
-  // otherwise a ~= a[$-i] won't work correctly
+  // Evaluate the expression to be appended first; it may affect the array.
   DValue *expVal = toElem(exp);
 
+  // The druntime function extends the slice in-place (length += 1, ptr
+  // potentially moved to a new block).
   LLFunction *fn = getRuntimeFunction(loc, gIR->module, "_d_arrayappendcTX");
-  LLValue *appendedArray =
-      gIR->CreateCallOrInvoke(
-             fn, DtoTypeInfoOf(arrayType),
-             DtoBitCast(DtoLVal(array), fn->getFunctionType()->getParamType(1)),
-             DtoConstSize_t(1), ".appendedArray")
-          .getInstruction();
-  appendedArray = DtoAggrPaint(appendedArray, DtoType(arrayType));
+  gIR->CreateCallOrInvoke(
+      fn, DtoTypeInfoOf(arrayType),
+      DtoBitCast(DtoLVal(array), fn->getFunctionType()->getParamType(1)),
+      DtoConstSize_t(1), ".appendedArray");
 
+  // Assign to the new last element.
+  LLValue *newLength = DtoArrayLen(array);
   LLValue *ptr = DtoArrayPtr(array);
-  ptr = DtoGEP1(ptr, oldLength, true, ".lastElem");
-  DLValue lastElem(arrayType->nextOf(), ptr);
+  LLValue *lastIndex =
+      gIR->ir->CreateSub(newLength, DtoConstSize_t(1), ".lastIndex");
+  LLValue *lastElemPtr = DtoGEP1(ptr, lastIndex, true, ".lastElem");
+  DLValue lastElem(arrayType->nextOf(), lastElemPtr);
   DtoAssign(loc, &lastElem, expVal, TOKblit);
-  callPostblit(loc, exp, ptr);
+  callPostblit(loc, exp, lastElemPtr);
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -1145,26 +1146,6 @@ LLValue *DtoArrayEquals(Loc &loc, TOK op, DValue *l, DValue *r) {
   return res;
 }
 
-////////////////////////////////////////////////////////////////////////////////
-LLValue *DtoArrayCompare(Loc &loc, TOK op, DValue *l, DValue *r) {
-  LLValue *res = nullptr;
-  llvm::ICmpInst::Predicate cmpop;
-  tokToICmpPred(op, false, &cmpop, &res);
-
-  if (!res) {
-    Type *t = l->type->toBasetype()->nextOf()->toBasetype();
-    if (t->ty == Tchar) {
-      res = DtoArrayEqCmp_impl(loc, "_adCmpChar", l, r, false);
-    } else {
-      res = DtoArrayEqCmp_impl(loc, "_adCmp2", l, r, true);
-    }
-    res = gIR->ir->CreateICmp(cmpop, res, DtoConstInt(0));
-  }
-
-  assert(res);
-  return res;
-}
-
 ////////////////////////////////////////////////////////////////////////////////
 LLValue *DtoArrayCastLength(Loc &loc, LLValue *len, LLType *elemty,
                             LLType *newelemty) {
@@ -1379,7 +1360,7 @@ void DtoIndexBoundsCheck(Loc &loc, DValue *arr, DValue *index) {
 void DtoBoundsCheckFailCall(IRState *irs, Loc &loc) {
   Module *const module = irs->func()->decl->getModule();
 
-  if (global.params.betterC) {
+  if (global.params.checkAction == CHECKACTION_C) {
     DtoCAssert(module, loc, DtoConstCString("array overflow"));
   } else {
     llvm::Function *errorfn =
