@@ -15,6 +15,7 @@
 #include "module.h"
 #include "scope.h"
 #include "template.h"
+#include "gen/inlineir.h"
 #include "gen/llvmhelpers.h"
 #include "llvm/Support/CommandLine.h"
 
@@ -54,7 +55,7 @@ LDCPragma DtoGetPragma(Scope *sc, PragmaDeclaration *decl,
   Identifier *ident = decl->ident;
   Expressions *args = decl->args;
   Expression *expr =
-      (args && args->dim > 0) ? semantic((*args)[0], sc) : nullptr;
+      (args && args->dim > 0) ? expressionSemantic((*args)[0], sc) : nullptr;
 
   // pragma(LDC_intrinsic, "string") { funcdecl(s) }
   if (ident == Id::LDC_intrinsic) {
@@ -98,8 +99,10 @@ LDCPragma DtoGetPragma(Scope *sc, PragmaDeclaration *decl,
     return LLVMintrinsic;
   }
 
-  // pragma(LDC_global_crt_ctor [, priority]) { funcdecl(s) }
-  if (ident == Id::LDC_global_crt_ctor || ident == Id::LDC_global_crt_dtor) {
+  // pragma(LDC_global_crt_{c,d}tor [, priority]) { funcdecl(s) }
+  // pragma(crt_{con,de}structor [, priority]) { funcdecl(s) }
+  if (ident == Id::LDC_global_crt_ctor || ident == Id::LDC_global_crt_dtor ||
+      ident == Id::crt_constructor || ident == Id::crt_destructor) {
     dinteger_t priority;
     if (args) {
       if (args->dim != 1 || !parseIntExp(expr, priority)) {
@@ -116,8 +119,9 @@ LDCPragma DtoGetPragma(Scope *sc, PragmaDeclaration *decl,
     char buf[8];
     sprintf(buf, "%llu", static_cast<unsigned long long>(priority));
     arg1str = strdup(buf);
-    return ident == Id::LDC_global_crt_ctor ? LLVMglobal_crt_ctor
-                                            : LLVMglobal_crt_dtor;
+    return ident == Id::LDC_global_crt_ctor || ident == Id::crt_constructor
+               ? LLVMglobal_crt_ctor
+               : LLVMglobal_crt_dtor;
   }
 
   // pragma(LDC_no_typeinfo) { typedecl(s) }
@@ -429,45 +433,11 @@ void DtoCheckPragma(PragmaDeclaration *decl, Dsymbol *s,
     break;
 
   case LLVMinline_ir:
-    if (TemplateDeclaration *td = s->isTemplateDeclaration()) {
-      Dsymbol *member = td->onemember;
-      if (!member) {
-        error(s->loc, "the `%s` pragma template must have exactly one member",
-              ident->toChars());
-        fatal();
-      }
-      FuncDeclaration *fun = member->isFuncDeclaration();
-      if (!fun) {
-        error(
-            s->loc,
-            "the `%s` pragma template's member must be a function declaration",
-            ident->toChars());
-        fatal();
-      }
-
-      TemplateParameters &params = *td->parameters;
-      bool valid_params = params.dim == 3 &&
-                          params[1]->isTemplateTypeParameter() &&
-                          params[2]->isTemplateTupleParameter();
-
-      if (valid_params) {
-        TemplateValueParameter *p0 = params[0]->isTemplateValueParameter();
-        valid_params = valid_params && p0 && p0->valType == Type::tstring;
-      }
-
-      if (!valid_params) {
-        error(s->loc,
-              "the `%s` pragma template must have exactly three parameters: "
-              "a string, a type and a type tuple",
-              ident->toChars());
-        fatal();
-      }
-
+    DtoCheckInlineIRPragma(ident, s);
+    {
+      TemplateDeclaration *td = s->isTemplateDeclaration();
+      assert(td != nullptr);
       td->llvmInternal = llvm_internal;
-    } else {
-      error(s->loc, "the `%s` pragma is only allowed on template declarations",
-            ident->toChars());
-      fatal();
     }
     break;
 
