@@ -239,14 +239,8 @@ public:
       // adding more control flow.
       if (result && result->type->ty != Tvoid &&
           !result->definedInFuncEntryBB()) {
-        if (result->isLVal()) {
-          LLValue *copy = DtoAlloca(result->type);
-          DtoMemCpy(copy, DtoLVal(result));
-          result = new DLValue(result->type, copy);
-        } else {
-          LLValue *copy = DtoAllocaDump(result);
-          result = new DLValue(result->type, copy);
-        }
+        LLValue *copy = DtoAllocaDump(result);
+        result = new DLValue(result->type, copy);
       }
 
       llvm::BasicBlock *endbb = p->insertBB("toElem.success");
@@ -597,9 +591,24 @@ public:
 
   using BinOpFunc = DValue *(Loc &, Type *, DValue *, Expression *, bool);
 
+  static Expression *getLValExp(Expression *e) {
+    e = skipOverCasts(e);
+    if (e->op == TOKcomma) {
+      CommaExp *ce = static_cast<CommaExp *>(e);
+      Expression *newCommaRhs = getLValExp(ce->e2);
+      if (newCommaRhs != ce->e2) {
+        CommaExp *newComma = static_cast<CommaExp *>(ce->copy());
+        newComma->e2 = newCommaRhs;
+        newComma->type = newCommaRhs->type;
+        e = newComma;
+      }
+    }
+    return e;
+  }
+
   template <BinOpFunc binOpFunc, bool useLValTypeForBinOp>
   static DValue *binAssign(BinAssignExp *e) {
-    Expression *lvalExp = skipOverCasts(e->e1);
+    Expression *lvalExp = getLValExp(e->e1);
     DValue *lhsLVal = toElem(lvalExp);
 
     // Use the lhs lvalue for the binop lhs and optionally cast it to the full
@@ -1012,20 +1021,19 @@ public:
     assert(!isSpecialRefVar(vd) && "Code not expected to handle special ref "
                                    "vars, although it can easily be made to.");
 
-    LLValue *v;
     const auto ident = p->func()->decl->ident;
     if (ident == Id::ensure || ident == Id::require) {
       Logger::println("contract this exp");
-      v = DtoBitCast(p->func()->nestArg, DtoType(e->type)->getPointerTo());
+      LLValue *v = DtoBitCast(p->func()->nestArg, DtoType(e->type));
+      result = new DImValue(e->type, v);
     } else if (vd->toParent2() != p->func()->decl) {
       Logger::println("nested this exp");
       result = DtoNestedVariable(e->loc, e->type, vd, e->type->ty == Tstruct);
-      return;
     } else {
       Logger::println("normal this exp");
-      v = p->func()->thisArg;
+      LLValue *v = p->func()->thisArg;
+      result = new DLValue(e->type, DtoBitCast(v, DtoPtrToType(e->type)));
     }
-    result = new DLValue(e->type, DtoBitCast(v, DtoPtrToType(e->type)));
   }
 
   //////////////////////////////////////////////////////////////////////////////
@@ -1262,9 +1270,6 @@ public:
         llvm_unreachable("Unsupported floating point comparison operator.");
       }
       eval = p->ir->CreateFCmp(cmpop, DtoRVal(l), DtoRVal(r));
-    } else if (t->ty == Tsarray || t->ty == Tarray) {
-      Logger::println("static or dynamic array");
-      eval = DtoArrayCompare(e->loc, e->op, l, r);
     } else if (t->ty == Taarray) {
       eval = LLConstantInt::getFalse(gIR->context());
     } else if (t->ty == Tdelegate) {
@@ -1604,7 +1609,9 @@ public:
       } else if (e->e1->op == TOKvar) {
         if (auto vd = static_cast<VarExp *>(e->e1)->var->isVarDeclaration()) {
           if (vd->onstack) {
-            DtoFinalizeClass(e->loc, DtoRVal(dval));
+            assert(vd->scopeClassType);
+            const auto cd = vd->scopeClassType->sym->isClassDeclaration();
+            DtoFinalizeScopeClass(e->loc, DtoRVal(dval), cd);
             onstack = true;
           }
         }
@@ -1654,7 +1661,7 @@ public:
     auto &PGO = gIR->funcGen().pgo;
     PGO.setCurrentStmt(e);
 
-    if (!global.params.useAssert)
+    if (global.params.useAssert != CHECKENABLEon)
       return;
 
     // condition
@@ -1686,7 +1693,7 @@ public:
      */
     DValue *const msg = e->msg ? toElemDtor(e->msg) : nullptr;
     Module *const module = p->func()->decl->getModule();
-    if (global.params.betterC) {
+    if (global.params.checkAction == CHECKACTION_C) {
       const auto cMsg =
           msg ? DtoArrayPtr(msg) // assuming `msg` is null-terminated, like DMD
               : DtoConstCString(e->e1->toChars());
@@ -1752,8 +1759,8 @@ public:
 
   //////////////////////////////////////////////////////////////////////////////
 
-  void visit(AndAndExp *e) override {
-    IF_LOG Logger::print("AndAndExp::toElem: %s @ %s\n", e->toChars(),
+  void visit(LogicalExp *e) override {
+    IF_LOG Logger::print("LogicalExp::toElem: %s @ %s\n", e->toChars(),
                          e->type->toChars());
     LOG_SCOPE;
 
@@ -1762,18 +1769,27 @@ public:
 
     DValue *u = toElem(e->e1);
 
-    llvm::BasicBlock *andand = p->insertBB("andand");
-    llvm::BasicBlock *andandend = p->insertBBAfter(andand, "andandend");
+    const bool isAndAnd = (e->op == TOKandand); // otherwise OrOr
+    llvm::BasicBlock *rhsBB = p->insertBB(isAndAnd ? "andand" : "oror");
+    llvm::BasicBlock *endBB =
+        p->insertBBAfter(rhsBB, isAndAnd ? "andandend" : "ororend");
 
     LLValue *ubool = DtoRVal(DtoCast(e->loc, u, Type::tbool));
 
     llvm::BasicBlock *oldblock = p->scopebb();
-    auto truecount = PGO.getRegionCount(e);
-    auto falsecount = PGO.getCurrentRegionCount() - truecount;
+    uint64_t truecount, falsecount;
+    if (isAndAnd) {
+      truecount = PGO.getRegionCount(e);
+      falsecount = PGO.getCurrentRegionCount() - truecount;
+    } else {
+      falsecount = PGO.getRegionCount(e);
+      truecount = PGO.getCurrentRegionCount() - falsecount;
+    }
     auto branchweights = PGO.createProfileWeights(truecount, falsecount);
-    p->ir->CreateCondBr(ubool, andand, andandend, branchweights);
+    p->ir->CreateCondBr(ubool, isAndAnd ? rhsBB : endBB,
+                        isAndAnd ? endBB : rhsBB, branchweights);
 
-    p->scope() = IRScope(andand);
+    p->scope() = IRScope(rhsBB);
     PGO.emitCounterIncrement(e);
     emitCoverageLinecountInc(e->e2->loc);
     DValue *v = toElemDtor(e->e2);
@@ -1784,8 +1800,8 @@ public:
     }
 
     llvm::BasicBlock *newblock = p->scopebb();
-    llvm::BranchInst::Create(andandend, p->scopebb());
-    p->scope() = IRScope(andandend);
+    llvm::BranchInst::Create(endBB, p->scopebb());
+    p->scope() = IRScope(endBB);
 
     // DMD allows stuff like `x == 0 && assert(false)`
     if (e->type->toBasetype()->ty == Tvoid) {
@@ -1799,70 +1815,17 @@ public:
       resval = ubool;
     } else {
       llvm::PHINode *phi =
-          p->ir->CreatePHI(LLType::getInt1Ty(gIR->context()), 2, "andandval");
-      // If we jumped over evaluation of the right-hand side,
-      // the result is false. Otherwise it's the value of the right-hand side.
-      phi->addIncoming(LLConstantInt::getFalse(gIR->context()), oldblock);
-      phi->addIncoming(vbool, newblock);
-      resval = phi;
-    }
-
-    result = new DImValue(e->type, resval);
-  }
-
-  //////////////////////////////////////////////////////////////////////////////
-
-  void visit(OrOrExp *e) override {
-    IF_LOG Logger::print("OrOrExp::toElem: %s @ %s\n", e->toChars(),
-                         e->type->toChars());
-    LOG_SCOPE;
-
-    auto &PGO = gIR->funcGen().pgo;
-    PGO.setCurrentStmt(e);
-
-    DValue *u = toElem(e->e1);
-
-    llvm::BasicBlock *oror = p->insertBB("oror");
-    llvm::BasicBlock *ororend = p->insertBBAfter(oror, "ororend");
-
-    LLValue *ubool = DtoRVal(DtoCast(e->loc, u, Type::tbool));
-
-    llvm::BasicBlock *oldblock = p->scopebb();
-    auto falsecount = PGO.getRegionCount(e);
-    auto truecount = PGO.getCurrentRegionCount() - falsecount;
-    auto branchweights = PGO.createProfileWeights(truecount, falsecount);
-    p->ir->CreateCondBr(ubool, ororend, oror, branchweights);
-
-    p->scope() = IRScope(oror);
-    PGO.emitCounterIncrement(e);
-    emitCoverageLinecountInc(e->e2->loc);
-    DValue *v = toElemDtor(e->e2);
-
-    LLValue *vbool = nullptr;
-    if (v && !v->isFunc() && v->type != Type::tvoid) {
-      vbool = DtoRVal(DtoCast(e->loc, v, Type::tbool));
-    }
-
-    llvm::BasicBlock *newblock = p->scopebb();
-    llvm::BranchInst::Create(ororend, p->scopebb());
-    p->scope() = IRScope(ororend);
-
-    // DMD allows stuff like `x == 0 || assert(false)`
-    if (e->type->toBasetype()->ty == Tvoid) {
-      result = nullptr;
-      return;
-    }
-
-    LLValue *resval = nullptr;
-    if (ubool == vbool || !vbool) {
-      // No need to create a PHI node.
-      resval = ubool;
-    } else {
-      llvm::PHINode *phi =
-          p->ir->CreatePHI(LLType::getInt1Ty(gIR->context()), 2, "ororval");
-      // If we jumped over evaluation of the right-hand side,
-      // the result is true. Otherwise, it's the value of the right-hand side.
-      phi->addIncoming(LLConstantInt::getTrue(gIR->context()), oldblock);
+          p->ir->CreatePHI(LLType::getInt1Ty(gIR->context()), 2,
+                           isAndAnd ? "andandval" : "ororval");
+      if (isAndAnd) {
+        // If we jumped over evaluation of the right-hand side,
+        // the result is false. Otherwise it's the value of the right-hand side.
+        phi->addIncoming(LLConstantInt::getFalse(gIR->context()), oldblock);
+      } else {
+        // If we jumped over evaluation of the right-hand side,
+        // the result is true. Otherwise, it's the value of the right-hand side.
+        phi->addIncoming(LLConstantInt::getTrue(gIR->context()), oldblock);
+      }
       phi->addIncoming(vbool, newblock);
       resval = phi;
     }
@@ -2163,7 +2126,7 @@ public:
       DtoStore(DtoRVal(slice), DtoLVal(result));
     } else {
       // append element
-      DtoCatAssignElement(e->loc, e1type, result, e->e2);
+      DtoCatAssignElement(e->loc, result, e->e2);
     }
   }
 
@@ -2220,7 +2183,7 @@ public:
         // function. Happens with anonymous functions.
         cval = funcGen.nestedVar;
       } else if (irfn.nestArg) {
-        cval = DtoLoad(irfn.nestArg);
+        cval = irfn.nestArg;
       } else if (irfn.thisArg) {
         AggregateDeclaration *ad = irfn.decl->isMember2();
         if (!ad || !ad->vthis) {
@@ -2435,7 +2398,7 @@ public:
       // Turn it back into a TypeAArray
       vtype = e->values->tdata()[0]->type;
       aatype = TypeAArray::create(vtype, e->keys->tdata()[0]->type);
-      aatype = aatype->semantic(e->loc, nullptr);
+      aatype = typeSemantic(aatype, e->loc, nullptr);
     }
 
     {
