@@ -56,7 +56,7 @@
 
 FuncDeclaration *search_toString(StructDeclaration *sd);
 
-// defined in ddmd/typinf.d:
+// defined in dmd/typinf.d:
 void genTypeInfo(Type *torig, Scope *sc);
 bool builtinTypeInfo(Type *t);
 
@@ -435,7 +435,7 @@ public:
       Type *t = sd->arg1type;
       for (unsigned i = 0; i < 2; i++) {
         if (t) {
-          t = t->merge();
+          t = merge(t);
           b.push_typeinfo(t);
         } else {
           b.push_null(Type::dtypeinfo->type);
@@ -532,7 +532,7 @@ public:
 
     RTTIBuilder b(Type::typeinfoconst);
     // TypeInfo base
-    b.push_typeinfo(decl->tinfo->mutableOf()->merge());
+    b.push_typeinfo(merge(decl->tinfo->mutableOf()));
     // finish
     b.finalize(gvar);
   }
@@ -546,7 +546,7 @@ public:
 
     RTTIBuilder b(Type::typeinfoinvariant);
     // TypeInfo base
-    b.push_typeinfo(decl->tinfo->mutableOf()->merge());
+    b.push_typeinfo(merge(decl->tinfo->mutableOf()));
     // finish
     b.finalize(gvar);
   }
@@ -560,7 +560,7 @@ public:
 
     RTTIBuilder b(Type::typeinfoshared);
     // TypeInfo base
-    b.push_typeinfo(decl->tinfo->unSharedOf()->merge());
+    b.push_typeinfo(merge(decl->tinfo->unSharedOf()));
     // finish
     b.finalize(gvar);
   }
@@ -574,7 +574,7 @@ public:
 
     RTTIBuilder b(Type::typeinfowild);
     // TypeInfo base
-    b.push_typeinfo(decl->tinfo->mutableOf()->merge());
+    b.push_typeinfo(merge(decl->tinfo->mutableOf()));
     // finish
     b.finalize(gvar);
   }
@@ -638,7 +638,7 @@ void TypeInfoDeclaration_codegen(TypeInfoDeclaration *decl, IRState *p) {
   emitTypeMetadata(decl);
 
   // check if the definition can be elided
-  if (global.params.betterC || isSpeculativeType(decl->tinfo) ||
+  if (!global.params.useTypeInfo || isSpeculativeType(decl->tinfo) ||
       builtinTypeInfo(decl->tinfo)) {
     return;
   }
