@@ -16,20 +16,26 @@
 
 struct Context;
 
+#define MAKE_JIT_API_CALL_IMPL(prefix, version) prefix##version
+#define MAKE_JIT_API_CALL(prefix, version)                                     \
+  MAKE_JIT_API_CALL_IMPL(prefix, version)
+#define JIT_API_ENTRYPOINT                                                     \
+  MAKE_JIT_API_CALL(rtCompileProcessImplSo, LDC_DYNAMIC_COMPILE_API_VERSION)
+
 extern "C" {
 
 // Silence missing-variable-declaration clang warning
-extern void *dynamiccompile_modules_head;
+extern const void *dynamiccompile_modules_head;
 
-void *dynamiccompile_modules_head = nullptr;
+const void *dynamiccompile_modules_head = nullptr;
 #ifdef _WIN32
 __declspec(dllimport)
 #endif
-    extern void rtCompileProcessImplSo(const void *modlist_head,
-                                       const Context *context,
-                                       std::size_t contextSize);
+extern void JIT_API_ENTRYPOINT(const void *modlist_head,
+                               const Context *context,
+                               std::size_t contextSize);
 
 void rtCompileProcessImpl(const Context *context, std::size_t contextSize) {
-  rtCompileProcessImplSo(dynamiccompile_modules_head, context, contextSize);
+  JIT_API_ENTRYPOINT(dynamiccompile_modules_head, context, contextSize);
 }
 }
